Class: Feed2Imap

Inherits:
Object
  • Object
show all
Defined in:
lib/feed2imap/version.rb,
lib/feed2imap/feed2imap.rb

Constant Summary collapse

VERSION =
"1.3.5"

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(verbose, cacherebuild, configfile) ⇒ Feed2Imap

Returns a new instance of Feed2Imap.



38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
# File 'lib/feed2imap/feed2imap.rb', line 38

def initialize(verbose, cacherebuild, configfile)
  @logger = Logger::new(STDOUT)
  if verbose == :debug
    @logger.level = Logger::DEBUG
    require 'pp'
  elsif verbose == true
    @logger.level = Logger::INFO
  else
    @logger.level = Logger::WARN
  end
  @logger.info("Feed2Imap V.#{Feed2Imap::VERSION} started")
  # reading config
  @logger.info('Reading configuration file ...')
  if not File::exist?(configfile)
    @logger.fatal("Configuration file #{configfile} not found.")
    exit(1)
  end
  if (File::stat(configfile).mode & 044) != 0
    @logger.warn("Configuration file is readable by other users. It " +
      "probably contains your password.")
  end
  begin
    File::open(configfile) { 
      |f| @config = F2IConfig::new(f)
    }
  rescue
    @logger.fatal("Error while reading configuration file, exiting: #{$!}")
    exit(1)
  end
  if @logger.level == Logger::DEBUG
    @logger.debug("Configuration read:")
    pp(@config)
  end

  # init cache
  @logger.info('Initializing cache ...')
  @cache = ItemCache::new(@config.updateddebug)
  if not File::exist?(@config.cache + '.lock')
    f = File::new(@config.cache + '.lock', 'w')
    f.close
  end
  if File::new(@config.cache + '.lock', 'w').flock(File::LOCK_EX | File::LOCK_NB) == false
    @logger.fatal("Another instance of feed2imap is already locking the cache file")
    exit(1)
  end
  if not File::exist?(@config.cache) 
    @logger.warn("Cache file #{@config.cache} not found, using a new one")
  else
    File::open(@config.cache) do |f|
      @cache.load(f)
    end
  end

  # connecting all IMAP accounts
  @logger.info('Connecting to IMAP accounts ...')
  @config.imap_accounts.each_value do |ac|
    begin
      ac.connect
    rescue
      @logger.fatal("Error while connecting to #{ac}, exiting: #{$!}")
      exit(1)
    end
  end

  # for each feed, fetch, upload to IMAP and cache
  @logger.info("Fetching and filtering feeds ...")
  ths = []
  mutex = Mutex::new
  sparefetchers = 16 # max number of fetchers running at the same time.
  sparefetchers_mutex = Mutex::new
  sparefetchers_cond = ConditionVariable::new
  @config.feeds.each do |f|
    ths << Thread::new(f) do |feed|
      begin
        mutex.lock
        lastcheck = @cache.get_last_check(feed.name) 
        if feed.needfetch(lastcheck)
          mutex.unlock
          sparefetchers_mutex.synchronize do
            while sparefetchers <= 0
              sparefetchers_cond.wait(sparefetchers_mutex)
            end
            sparefetchers -= 1
          end
          fetch_start = Time::now
          if feed.url
            fetcher = HTTPFetcher::new
            fetcher::timeout = @config.timeout
            s = fetcher::fetch(feed.url, @cache.get_last_check(feed.name))
          elsif feed.execurl
            # avoid running more than one command at the same time.
            # We need it because the called command might not be
            # thread-safe, and we need to get the right exitcode
            mutex.lock
            s = %x{#{feed.execurl}}
            if $? && $?.exitstatus != 0
              @logger.warn("Command for #{feed.name} exited with status #{$?.exitstatus} !")
            end
            mutex.unlock
          else
            @logger.warn("No way to fetch feed #{feed.name} !")
          end
          if feed.filter and s != nil
            # avoid running more than one command at the same time.
            # We need it because the called command might not be
            # thread-safe, and we need to get the right exitcode.
            mutex.lock
            # hack hack hack, avoid buffering problems
            begin
              stdin, stdout, stderr = Open3::popen3(feed.filter)
              inth = Thread::new do
                stdin.puts s
                stdin.close
              end
              output = nil
              outh = Thread::new do
                output = stdout.read
              end
              err = nil
              errth = Thread::new do
                err = stderr.read.chomp
              end
              inth.join
              outh.join
              errth.join
              err.lines do |e|
                @logger.warn(e.chomp)
              end
              s = output
              if $? && $?.exitstatus != 0
                @logger.warn("Filter command for #{feed.name} exited with status #{$?.exitstatus}. Output might be corrupted !")
              end
            ensure
              mutex.unlock
            end
          end
          if Time::now - fetch_start > F2I_WARNFETCHTIME
            @logger.info("Fetching feed #{feed.name} took #{(Time::now - fetch_start).to_i}s")
          end
          sparefetchers_mutex.synchronize do
            sparefetchers += 1
            sparefetchers_cond.signal
          end
          mutex.lock
          feed.body = s
          @cache.set_last_check(feed.name, Time::now)
        else
          @logger.debug("Feed #{feed.name} doesn't need to be checked again for now.")
        end
        mutex.unlock
        # dump if requested
        if @config.dumpdir
          mutex.synchronize do
            if feed.body
              fname = @config.dumpdir + '/' + feed.name + '-' + Time::now.xmlschema
              File::open(fname, 'w') { |file| file.puts feed.body }
            end
          end
        end
        # dump this feed if requested
        if feed.dumpdir
          mutex.synchronize do
            if feed.body
              fname = feed.dumpdir + '/' + feed.name + '-' + Time::now.xmlschema
              File::open(fname, 'w') { |file| file.puts feed.body }
            end
          end
        end
      rescue Timeout::Error
        mutex.synchronize do
          n = @cache.fetch_failed(feed.name)
          m = "Timeout::Error while fetching #{feed.url}: #{$!} (failed #{n} times)"
          if n > @config.max_failures
            @logger.fatal(m)
          else
            @logger.info(m)
          end
        end
      rescue
        mutex.synchronize do
          n = @cache.fetch_failed(feed.name)
          m = "Error while fetching #{feed.url}: #{$!} (failed #{n} times)"
          if n > @config.max_failures
            @logger.fatal(m)
          else
            @logger.info(m)
          end
        end
      end
    end
  end
  ths.each { |t| t.join }
  @logger.info("Parsing and uploading ...")
  @config.feeds.each do |f|
    if f.body.nil? # means 304
      @logger.debug("Feed #{f.name} did not change.")
      next
    end
    begin
      feed = FeedParser::Feed::new(f.body.force_encoding('UTF-8'), f.url)
    rescue Exception
      n = @cache.parse_failed(f.name)
      m = "Error while parsing #{f.name}: #{$!} (failed #{n} times)"
      if n > @config.max_failures
        @logger.fatal(m)
      else
        @logger.info(m)
      end
      next
    end
    begin
      newitems, updateditems = @cache.get_new_items(f.name, feed.items, f.always_new, f.ignore_hash)
    rescue
      @logger.fatal("Exception caught when selecting new items for #{f.name}: #{$!}")
      puts $!.backtrace
      next
    end
    @logger.info("#{f.name}: #{newitems.length} new items, #{updateditems.length} updated items.") if newitems.length > 0 or updateditems.length > 0 or @logger.level == Logger::DEBUG
    begin
      if !cacherebuild
        fn = f.name.gsub(/[^0-9A-Za-z]/,'')
        updateditems.each do |i|
          id = "<#{fn}-#{i.cacheditem.index}@#{@config.hostname}>"
          email = item_to_mail(@config, i, id, true, f.name, f.include_images, f.wrapto)
          f.imapaccount.updatemail(f.folder, email,
                                   id, i.date || Time::new, f.reupload_if_updated)
        end
        # reverse is needed to upload older items first (fixes gna#8986)
        newitems.reverse.each do |i|
          id = "<#{fn}-#{i.cacheditem.index}@#{@config.hostname}>"
          email = item_to_mail(@config, i, id, false, f.name, f.include_images, f.wrapto)
          f.imapaccount.putmail(f.folder, email, i.date || Time::new)
        end
      end
    rescue
      @logger.fatal("Exception caught while uploading mail to #{f.folder}: #{$!}")
      puts $!.backtrace
      @logger.fatal("We can't recover from IMAP errors, so we are exiting.")
      exit(1)
    end
    begin
      @cache.commit_cache(f.name)
    rescue
      @logger.fatal("Exception caught while updating cache for #{f.name}: #{$!}")
      next
    end
  end
  @logger.info("Finished. Saving cache ...")
  begin
    File::open("#{@config.cache}.new", 'w') { |f| @cache.save(f) }
  rescue
    @logger.fatal("Exception caught while writing new cache to #{@config.cache}.new: #{$!}")
  end
  begin
    File::rename("#{@config.cache}.new", @config.cache)
  rescue
    @logger.fatal("Exception caught while renaming #{@config.cache}.new to #{@config.cache}: #{$!}")
  end
  @logger.info("Closing IMAP connections ...")
  @config.imap_accounts.each_value do |ac|
    begin
      ac.disconnect
    rescue
      # servers tend to cause an exception to be raised here, hence the INFO level.
      @logger.info("Exception caught while closing connection to #{ac.to_s}: #{$!}")
    end
  end
end

Class Method Details

.versionObject



34
35
36
# File 'lib/feed2imap/feed2imap.rb', line 34

def Feed2Imap.version
  return Feed2Imap::VERSION
end